==> billiards-0.1.tar.gz: billiards-0.1/src/item.c <==
/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include "item.h"

@implementation Item

-(Item *) init
{
    [super init];

    self->up = NULL;
    self->down = NULL;
    self->left = NULL;
    self->right = NULL;

    return self;
}

-(id) parent
{
    return self->up;
}

-(id) children
{
    return self->down;
}

-(id) brother
{
    return self->left;
}

-(id) sister
{
    return self->right;
}

-(id) root
{
    id root;
    
    for (root = self ; [root parent] ; root = [root parent]);

    return (id)root;
}

-(void) adopt: (Item *)child named: (char *)name
{
    Item *sibling;
    
    /* We can't adopt her if she
       allready has a parent. */

    if (child->up == nil) {
	child->key = name;
	child->up = self;

	if (!self->down) {
	    self->down = child;
	    assert (!child->left && !child->right);
	} else {
	    for (sibling = self->down;
		 sibling->right && strcmp(sibling->right->key, child->key) < 0;
		 sibling = sibling->right);

	    if(strcmp(sibling->key, child->key) < 0) {
		child->left = sibling;
		child->right = sibling->right;
		sibling->right = child;
		
		if (child->right) {
		    child->right->left = child;
		}
	    } else {
		assert(self->down == sibling);

		self->down = child;
		child->right = sibling;
		sibling->left = child;
	    }

	    assert (!child->left || strcmp(child->left->key, child->key) <= 0);
	    assert (!child->right || strcmp(child->right->key, child->key) >= 0);
	}    
    }
}

-(void) renounce: (Item *)child
{
    if (child->right) {
	child->right->left = child->left;
    }
	    
    if (child->left){
	child->left->right = child->right;
    } else {
	self->down = child->right;
    }

    child->left = NULL;
    child->right = NULL;
    child->up = NULL;
}

@end

==> billiards-0.2.1.tar.gz: billiards-0.2.1/src/item.c <==
/* Copyright (C) 2008 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include "item.h"

@implementation Item

-forward: (SEL) sel : args
{
    return nil;
}

-(Item *) init
{
    [super init];

    self->up = NULL;
    self->down = NULL;
    self->left = NULL;
    self->right = NULL;

    return self;
}

-(char *) key
{
    return self->key;
}

-(id) parent
{
    return self->up;
}

-(id) children
{
    return self->down;
}

-(id) brother
{
    return self->left;
}

-(id) sister
{
    return self->right;
}

-(id) root
{
    id root;
    
    for (root = self ; [root parent] ; root = [root parent]);

    return (id)root;
}

-(void) adoptedBy: (Item *)parent
{
}

-(void) adopt: (Item *)child named: (char *)name
{
    Item *sibling;
    
    child->key = name;
    child->up = self;

    if (!self->down) {
	self->down = child;
	assert (!child->left && !child->right);
    } else {
	for (sibling = self->down;
	     sibling->right && xstrcmp(sibling->right->key, [child key]) < 0;
	     sibling = sibling->right);

	if(xstrcmp(sibling->key, [child key]) < 0) {
	    child->left = sibling;
	    child->right = sibling->right;
	    sibling->right = child;
		
	    if (child->right) {
		child->right->left = child;
	    }
	} else {
	    assert(self->down == sibling);

	    self->down = child;
	    child->right = sibling;
	    sibling->left = child;
	}

	assert (!child->left || xstrcmp(child->left->key, [child key]) <= 0);
	assert (!child->right || xstrcmp(child->right->key, [child key]) >= 0);
    }    
}

-(void) renounce: (Item *)child
{
    if (child->right) {
	child->right->left = child->left;
    }
	    
    if (child->left){
	child->left->right = child->right;
    } else {
	self->down = child->right;
    }

    child->left = NULL;
    child->right = NULL;
    child->up = NULL;
}

@end
